// file: hmm_init.cc
//

// system include files
//
#include <stdlib.h>
#include <memory.h>

// isip include files
//
#include "../include/hmm.h"

#include "../include/integral_constants.h"

//-----------------------------------------------------------------------------
// method: init_cc
//
// arguments:
//
//  int_4 new_S: (input) number of states used in the model
//  int_4 new_K: (input) number of output symbols in the model
//
// return value: a logical_1 indicating status
//
// this function initalizes a model by allocating internal space
//
//-----------------------------------------------------------------------------
logical_1 Hmm::init_cc(int_4 new_S, int_4 new_K) {

  // cleanup
  //
  if (cleanup_cc() == ISIP_FALSE)
    {return ISIP_FALSE;}

  // allocate space
  //
  if (allocate_cc(new_S, new_K) == ISIP_FALSE)
    {return ISIP_FALSE;}

  // exit gracefully
  //
  return ISIP_TRUE;
}

//-----------------------------------------------------------------------------
// method: init_counters_cc
//
// arguments: none
//
// return value: a logical_1 indicating status
//
// this function initalizes counters used in the training pass
//
//-----------------------------------------------------------------------------
logical_1 Hmm::init_counters_cc() {

  // free old space
  //
  if (cleanup_counters_cc() == ISIP_FALSE)
    {return ISIP_FALSE;}

  // allocate new space - initialize to zero
  //
  if (allocate_counters_cc() == ISIP_FALSE)
    {return ISIP_FALSE;}

  // exit gracefully
  //
  return ISIP_TRUE;
}

//-----------------------------------------------------------------------------
// method: seed_cc
//
// arguments: none
//
// return value: a logical_1 indicating status
//
// this function initalizes a model by randomly assigning weights
//
// we use a simple approach where a value of one is successively divided
//
//-----------------------------------------------------------------------------
logical_1 Hmm::seed_cc() {

  // assign transition probs
  //
  for (int_4 i=0; i<S; i++) {

    float_4 sum = 1.0;

    for (int_4 j=0; j<S; j++) {
      
      // generate a random number
      //
      float_4 weight = drand48();

      // assign a transition
      //
      if (j == (S-1))
	{a[i][j] = sum;}
      else {
	float_4 value = weight * sum;
	a[i][j] = value;
	sum -= value;
      }
    }
  }

  // assign output probabilities
  //
  for (int_4 i=0; i<S; i++) {

    float_4 sum = 1.0;

    for (int_4 j=0; j<K; j++) {
      
      // generate a random number
      //
      float_4 weight = drand48();

      // assign a transition
      //
      if (j == (K-1))
	{b[i][j] = sum;}
      else {
	float_4 value = weight * sum;
	b[i][j] = value;
	sum -= value;
      }
    }
  }

  // assign output probabilities
  //
  float_4 sum = 1.0;

  for (int_4 i=0; i<S; i++) {
      
    // generate a random number
    //
    float_4 weight = drand48();

    // assign a transition
    //
    if (i == (S-1))
      {pi[i] = sum;}
    else {
      float_4 value = weight * sum;
      pi[i] = value;
      sum -= value;
    }
  }

  // exit gracefully
  //
  return ISIP_TRUE;
}
