// file: hmm_oper.cc
//
// simple operators
//

// isip include files
//
#include "../include/hmm.h"

#include "../include/integral_constants.h"

//-----------------------------------------------------------------------------
// method: log10_cc
//
// arguments:
//  float_4 value: (input) value to be converted
//
// return: a float_4 value denoting the base 10 logarithm of value
//
// this method computes the base 10 logarithm with a simple argument check.
//-----------------------------------------------------------------------------
float_4 Hmm::log10_cc(float_4 value) {

  // check for zero
  //
  if (value <= 0.0) {
    return -9e09;
  }

  // return log10
  //
  return log10(value);
}

//-----------------------------------------------------------------------------
// method: inv_log10_cc
//
// arguments:
//  float_4 value: (input) value to be converted
//
// return: a float_4 value denoting the inverse base 10 logarithm of value
//
// this method computes the inverse log.
//-----------------------------------------------------------------------------
float_4 Hmm::inv_log10_cc(float_4 value) {

  // return the value
  //
  return pow(10.0, value);
}

//-----------------------------------------------------------------------------
// method: invert_cc
//
// arguments:
//  float_4 value: (input) value to be converted
//
// return: a float_4 value containing the inverse
//
// this method computes (1/x).
//-----------------------------------------------------------------------------
float_4 Hmm::invert_cc(float_4 value) {

  // check the value
  //
  if (value == (float_4)0.0)
    {return (float_4)1.0;}
  else
    {return 1.0/value;}
}

//-----------------------------------------------------------------------------
// method: match_cc
//
// arguments:
//  char_1* value: (input) value to be converted
//
// return: an int_4 value denoting the position of value in the symbol table
//
// this method performs a simple linear table lookup
//-----------------------------------------------------------------------------
int_4 Hmm::match_cc(char_1 value) {

  // declare local variables
  //
  int_4 pos = (int_4)-1;

  // try to find it in the current list
  //
  for (int_4 i=0; i<K; i++) {
    if (y[i] == value) {
      pos = i;
      return pos;
    }
  }

  // exit gracefully
  //
  return pos;
}
